#include <htc.h>
#include "i2c.h"

#define SDA		(RC2)
#define SCL		(RC1)
#define SDAIN()	(TRISC2 = 1)
#define SDAOUT()	(TRISC2 = 0)

#define wait_us() \
{ \
	int i; \
	for(i=0; i<1; i++) continue; \
}

void i2c_init()
{
	// Setup port direction.
	TRISC0 = 0;	// Reset (RC0)
	RC0 = 0;
	TRISC1 = 0;	// SCL output (RC1)
	TRISC2 = 0;	// SDA (i/o) but now out only (RC2)
	RC1 = 1;
	RC2 = 1;
	wait_us();
	RC0 = 1;		// /Reset 
	wait_us();
}

static void i2c_send_start(void)
{
	SDA = 0;
	wait_us();
	SCL = 0;
	wait_us();
}

static void i2c_send_stop(void)
{
	wait_us();
	SCL = 1;
	wait_us();
	SDA = 1;
	wait_us();
}

#define i2c_send_0() \
{ \
	SDA = 0;		\
	wait_us();	\
	SCL = 1;		\
	wait_us();	\
	SCL = 0;		\
}

#define i2c_send_1() \
{ \
	SDA = 1;		\
	wait_us();	\
	SCL = 1;		\
	wait_us();	\
	SCL = 0;		\
}

static unsigned char i2c_read_ack(void)
{
	unsigned char rv;
	SDAIN();
	wait_us();
	SCL = 1;
	wait_us();
	rv = SDA;
	SCL = 0;
	SDAOUT();
	return(rv);
}

// Start I2C frame with specified addr
void i2c_start(unsigned char addr)
{
	int i;

	// Send Start
	i2c_send_start();

	// Send address
	for(i = 0; i < 7; i++){
		if(addr & 0x40){
			i2c_send_1();
		}
		else
		{
			i2c_send_0();
		}
		addr <<= 1;
	}

	// Send R/W (R=1,W=0);
	i2c_send_0();

	// Read ack
	i2c_read_ack();
}

// Stop I2C frame
void i2c_stop()
{
	// Send stop
	i2c_send_stop();
	wait_us();
}

// Send data
void i2c_write(unsigned char data)
{
	int i;
	for(i=0; i < 8; i++){
		if(data & 0x80){
			i2c_send_1();
		} else {
			i2c_send_0();
		}
		data <<= 1;
	}
	// ACK
	i2c_read_ack();
}
