#include <htc.h>
#include "common.h"
#include "lcd.h"
#include "gps.h"
#include "i2c.h"
#include "serial.h"
#include "tiny_printf.h"
#include "ad.h"
#include "spi.h"
#include "fs.h"
#include "kmllog.h"

// See pic18f4550.h header
__CONFIG(1, CPUDIV1 & HSPLL & FCMDIS);	// HSPLL, HS 
__CONFIG(2, PWRTDIS & BORDIS & WDTDIS);
__CONFIG(3, PBDIGITAL & MCLREN);
__CONFIG(4, LVPDIS & DEBUGEN & ICPORTDIS); // Successfully attach debugger by PICKit3
//__CONFIG(4, DEBUGEN & XINSTDIS & LVPDIS & STVRDIS & ICPORTDIS);
__CONFIG(5, UNPROTECT);
__CONFIG(6, UNPROTECT);
__CONFIG(7, UNPROTECT);

static long numint;
		
void interrupt main_isr(void)
{
	numint ++;
	if(TMR0IF){
		gps_isr();
	}
}

#if 1
#define EN_INT() \
{ \
  GIE = 1; \
}
#else
#define EN_INT() \
{ \
  GIE =1; \
  asm("nop"); \
  asm("nop"); \
}
#endif

#define DI_INT()	(GIE=0)


char hoge_buff[32];
int hoge_i;
void hoge(char c)
{
	hoge_buff[hoge_i++] = c;
}

#define DISP_INIT		0
#define DISP_COORD		1
#define DISP_DAYTIME	2
#define DISP_ALTSPD	3
#define DISP_GPSSTAT	4
#define DISP_RESOURCE	5
#define DISP_FILEINFO	6
#define DISP_MIN		1
#define DISP_MAX		6

unsigned char disp_overlay_time;

void disp(unsigned char disp_id)
{
	unsigned char value;
	unsigned long kmllog_size;
	// Battery gauge (1LSB = 11.72mV)
	//  3.0V ... 255
	//  2.5V ... 210
     //  2.0V ... 180
	//  1.5V ... 125
	//  Batt level  cell voltage
	//         0      < 1.0
	//         1      < 1.2
	//         2      < 1.3
	//         3      > 1.3
	value = ad_getvalue(1);
	if(value < 180){
		lcd_puticon(ICON_BAT, 0);
	} else if(value < 205){
		lcd_puticon(ICON_BAT, 1);
	} else if(value < 230){
		lcd_puticon(ICON_BAT, 2);
	} else {
		lcd_puticon(ICON_BAT, 3);
	}
	lcd_puticon(ICON_ANT, gps_info.state);
	lcd_puticon(ICON_SCSI, kmllog_working);

	if(disp_overlay_time){
		disp_overlay_time --;
		return;
	}

	lcd_clear();
	switch(disp_id){
	case DISP_INIT:		// Startup
		LCD_LINE(0);
		tiny_printf(lcd_putc, NULL, "GPS Logger v0.00");
		LCD_LINE(1);
		tiny_printf(lcd_putc, NULL, "Please wait ... ");
		break;
	case DISP_COORD:	// Coordinate
		LCD_LINE(0);
		tiny_printf(lcd_putc, NULL, "%c%02hd.%02hd.%04hd %d",
			gps_info.latitude.polarity ? 'N' : 'S',
			gps_info.latitude.deg,
			gps_info.latitude.min,
			gps_info.latitude.submin,
			gps_info.numupdate);
		LCD_LINE(1);
		tiny_printf(lcd_putc, NULL, "%c%03hd.%02hd.%04hd %d",
			gps_info.longitude.polarity ? 'E' : 'W',
			gps_info.longitude.deg,
			gps_info.longitude.min,
			gps_info.longitude.submin,
			gps_info.numsat );
		break;
	case DISP_DAYTIME:	// Day and Time
		LCD_LINE(0);
		tiny_printf(lcd_putc, NULL, "Time: %02d:%02d:%02d", 
			gps_info.daytime.hour,
			gps_info.daytime.min,
			gps_info.daytime.sec );
		LCD_LINE(1);
		tiny_printf(lcd_putc, NULL, "Date: %02d/%02d/%02d",
			gps_info.daytime.year,
			gps_info.daytime.mon,
			gps_info.daytime.day );
		break;
	case DISP_ALTSPD:	// Altitude and Speed
		LCD_LINE(0);
		tiny_printf(lcd_putc, NULL, "ALT: %d.%d m", 
			gps_info.altitude.meter,
			gps_info.altitude.submeter);
		LCD_LINE(1);
		gps_info.speed.submeter = 0;
		gps_info.speed.meter = 0;
		tiny_printf(lcd_putc, NULL, "SPD: %hd.%hd km/h",
			gps_info.speed.meter,
			gps_info.speed.submeter/10);
		break;

	case DISP_GPSSTAT:
		LCD_LINE(0);
		tiny_printf(lcd_putc, NULL, "SAT: %d", gps_info.numsat );
		LCD_LINE(1);
		if(gps_info.hdop.meter == 99){
			tiny_printf(lcd_putc, NULL, "DOP: no data");
		} else {
			tiny_printf(lcd_putc, NULL, "DOP: %hd.%hd",
				gps_info.hdop.meter,
				gps_info.hdop.submeter );
		}
		break;

	case DISP_RESOURCE:	// Batt volt
		LCD_LINE(0);
		tiny_printf(lcd_putc, NULL, "BAT: %ld.%ld V      ",
			((long)value * 1172) / 100000,
			((long)value * 1172) % 100000 / 10000);

		break;
	case DISP_FILEINFO:	// Current file infomation
		if(kmllog_working){
			LCD_LINE(0);
			tiny_printf(lcd_putc, NULL, "%s", kmllog_name);
			kmllog_size = fs_get_size();
			LCD_LINE(1);
			if(kmllog_size < 1000){
				tiny_printf(lcd_putc, NULL, "SIZE: %ld B        ", kmllog_size);
			} else {
				tiny_printf(lcd_putc, NULL, "SIZE: %ld.%d kB    ", kmllog_size/1000, kmllog_size%1000/100);
			}
		} else {
			LCD_LINE(0);
			tiny_printf(lcd_putc, NULL, "Log not working ");
		}	
		break;
	}
}

void disp_status(const char *line1, const char *line2)
{
	lcd_clear();
	LCD_LINE(0);
	if(line1) lcd_puts(line1);
	LCD_LINE(1);
	if(line2) lcd_puts(line2);
	disp_overlay_time = 2;
}


#define SW_SHORT	1000
#define SW_LONG	50000

void main(void)
{
	unsigned long prev, sw_count;
	unsigned char disp_id, sw, sw_prev;

	TRISB5 = 0;
	memset((char *)&gps_info, 0, sizeof(gps_info));
	//gps_info.speed.meter = 0;
	//gps_info.speed.submeter = 0;
	lcd_init();
	gps_init();
	ser_init();
	ad_init();

	// Display initializing message
	disp(DISP_INIT);

	disp_id = DISP_COORD;

	EN_INT();
	// Infinite loop
	while(1){
		// Main loop speed monitor (RB5)
		if(RB5) RB5=0;
		else RB5=1;

		DI_INT();
		if(gps_info.numupdate != prev){			
			prev = gps_info.numupdate;
			EN_INT();
			// Enter here every second

			// Set measuring system to wgs-84
			if(gps_info.numupdate == 3){
				gps_setwgs84();
			}			

			// Write coordinate
			kmllog_point(0);

			// Update display
			disp(disp_id);
			continue;
		}
		prev = gps_info.numupdate;
		EN_INT();
		// Poll sw and take action
		sw = ad_getsw();
		if(sw != AD_NOSW && sw == sw_prev){
			sw_count ++;
		} else {
			if( sw_count > SW_SHORT && sw_count < SW_LONG){
				// Do short sw action
				if(sw_prev == AD_SW4){
					if(kmllog_working == 0){
						// Start logging
						kmllog_start();
					} else {
						// Marking
						kmllog_mark();
					}
						
				}
			
				if(sw_prev == AD_SW3){
					if(disp_id == DISP_MAX){
						disp_id = DISP_MIN;
					} else {
						disp_id ++;
					}
					disp(disp_id);
				}
			}
			sw_count = 0;
		}
		if(sw_count > SW_LONG){
			// Do long sw action
			if(sw_prev == AD_SW4){
				kmllog_stop();
			}
		}
		sw_prev = sw;
	}
}

