#include <stdarg.h>
#include "tiny_printf.h"

#define F_ZERO		0x01
#define F_SEQ			0x02
#define F_HASVALUE		0x04

// Place buffer in global to reduce auto vairables
// (Very limited with HITEC-C)
static char buff[16];
static char hextbl[] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};

// Use outfunc and/or dest to store result. 
void tiny_printf(void (*outfunc)(char), char *dest, const char *f, ...)
{
	va_list ap;
	
	char c, *vp;
	int width =0, mod;
	long value;
	char flags = 0;
	int len = 0;
	int dest_len = 0;
	va_start(ap, f);

	while(c = *f++){

		if(flags & F_SEQ){
			switch(c){
			case '0':
				flags |= F_ZERO;
				continue;
			case '1': case '2': case '3': case '4': case '5':
			case '6': case '7': case '8': case '9':
				width = c - '0';
				continue;
			case 'l':
				value = va_arg(ap, long);
				flags |= F_HASVALUE;
				continue;
			case 'h':
				value = va_arg(ap, short);
				flags |= F_HASVALUE;
				continue;
			case 'd':
				if( ~flags & F_HASVALUE){
					// assume given value is int
					value = va_arg(ap, int);
					flags |= F_HASVALUE;
				}
				if(value < 0){
					if(outfunc) outfunc('-');
					if(dest) dest[dest_len++] = '-';
				}
				if(value == 0){
					buff[0] = '0';
					len = 1;
				}
				while(value){
					mod = value % 10;
					value = value / 10;
					buff[len] = mod + '0';
					len++;
				}
				while(width-- > len){
					if(flags & F_ZERO){
						if(outfunc) outfunc('0');
						if(dest) dest[dest_len++] = '0';
					} else {
						if(outfunc) outfunc(' ');
						if(dest) dest[dest_len++] = ' ';
					}
				}
				do {
					len --;
					if(outfunc) outfunc(buff[len]);
					if(dest) dest[dest_len++] = buff[len];
			 	} while(len);
				flags &= ~F_SEQ;
				continue;
			case 'x':
				if( ~flags & F_HASVALUE){
					// assume given value is int
					value = va_arg(ap, int);
					flags |= F_HASVALUE;
				}
				if(value == 0){
					buff[0] = '0';
					len = 1;
				}
				while(value){
					mod = value % 16;
					value = ((unsigned long)value) >> 4;
					buff[len] = hextbl[mod];
					len ++;
				}
				while(width-- > len){
					if(flags & F_ZERO){
						if(outfunc) outfunc('0');
						if(dest) dest[dest_len++] = '0';
					} else {
						if(outfunc) outfunc(' ');
						if(dest) dest[dest_len++] = ' ';
					}
				}
				do {
					len --;
					if(outfunc) outfunc(buff[len]);
					if(dest) dest[dest_len++] = buff[len];
			 	} while(len);
				flags &= ~F_SEQ;
				continue;
			case 's':
				vp = va_arg(ap, char *);
				while(*vp){
					if(outfunc) outfunc(*vp);
					if(dest) dest[dest_len++] = *vp;
					vp++;
				}
				flags &= ~F_SEQ;
				continue;
			case 'c':
				value = va_arg(ap, int);
				if(outfunc) outfunc(value);
				if(dest) dest[dest_len++] = value;
				flags &= ~F_SEQ;
				continue;
			}
		}

		if(c == '%'){
			flags = F_SEQ;
			width = 0;
			len = 0;
			continue;

		} else {
			if(outfunc) outfunc(c);
			if(dest) dest[dest_len++] = c;
			continue;
		}
	}
	if(outfunc) outfunc(0);
	if(dest) dest[dest_len++] = 0;
	va_end(ap);
}
